SET NOCOUNT ON;
GO
CREATE DATABASE fn;
GO
USE fn;
GO
SELECT TOP (100000) 
  o1.[object_id], 
  o1.create_date
INTO dbo.src
FROM sys.all_objects AS o1
CROSS JOIN sys.all_objects AS o2
ORDER BY [object_id];
GO
CREATE CLUSTERED INDEX x ON dbo.src([object_id]);
GO
-- prime the cache
SELECT [object_id], create_date FROM dbo.src;
GO

CREATE PROCEDURE dbo.p_dt_Standard
  @dt_ CHAR(10) = NULL
AS
BEGIN
  SET NOCOUNT ON;
  SELECT @dt_ = CONVERT(CHAR(10), create_date, 120)
    FROM dbo.src
    ORDER BY [object_id];
END
GO
CREATE FUNCTION dbo.dt_Inline(@dt_ DATETIME)
RETURNS TABLE
AS
  RETURN (SELECT dt_ = CONVERT(CHAR(10), @dt_, 120));
GO
CREATE FUNCTION dbo.dt_Multi(@dt_ DATETIME)
RETURNS @t TABLE(dt_ CHAR(10))
AS
BEGIN
  INSERT @t(dt_) SELECT CONVERT(CHAR(10), @dt_, 120);
  RETURN;
END
GO
CREATE FUNCTION dbo.dt_Scalar(@dt_ DATETIME)
RETURNS CHAR(10)
AS
BEGIN
  RETURN (SELECT CONVERT(CHAR(10), @dt_, 120));
END
GO
CREATE PROCEDURE dbo.p_dt_Inline
  @dt_ CHAR(10) = NULL
AS
BEGIN
  SET NOCOUNT ON;
  SELECT @dt_ = dt.dt_
    FROM dbo.src AS o
    CROSS APPLY dbo.dt_Inline(o.create_date) AS dt
    ORDER BY o.[object_id];
END
GO
CREATE PROCEDURE dbo.p_dt_Multi
  @dt_ CHAR(10) = NULL
AS
BEGIN
  SET NOCOUNT ON;
  SELECT @dt_ = dt.dt_
    FROM dbo.src
    CROSS APPLY dbo.dt_Multi(create_date) AS dt
    ORDER BY [object_id];
END
GO
CREATE PROCEDURE dbo.p_dt_Scalar
  @dt_ CHAR(10) = NULL
AS
BEGIN
  SET NOCOUNT ON;
  SELECT @dt_ = dbo.dt_Scalar(create_date)
    FROM dbo.src
    ORDER BY [object_id];
END
GO
CREATE TABLE #t
(
  ID INT IDENTITY(1,1), 
  q VARCHAR(32), 
  s DATETIME2, 
  e DATETIME2
);
GO
INSERT #t(q,s) VALUES('p Standard',SYSDATETIME());
GO
EXEC dbo.p_dt_Standard;
GO 2
UPDATE #t SET e = SYSDATETIME() WHERE ID = 1;
GO
INSERT #t(q,s) VALUES('ad hoc Standard',SYSDATETIME());
GO
DECLARE @dt_st CHAR(10);
  SELECT @dt_st = CONVERT(CHAR(10), create_date, 120)
    FROM dbo.src
    ORDER BY [object_id];
GO 2
UPDATE #t SET e = SYSDATETIME() WHERE ID = 2;
GO
INSERT #t(q,s) VALUES('p Inline',SYSDATETIME());
GO
EXEC dbo.p_dt_Inline;
GO 2
UPDATE #t SET e = SYSDATETIME() WHERE ID = 3;
GO
INSERT #t(q,s) VALUES('ad hoc Inline',SYSDATETIME());
GO
DECLARE @dt_in CHAR(10);
  SELECT @dt_in = dt.dt_
    FROM dbo.src AS o
    CROSS APPLY dbo.dt_Inline(o.create_date) AS dt
    ORDER BY o.[object_id];
GO 2
UPDATE #t SET e = SYSDATETIME() WHERE ID = 4;
GO
INSERT #t(q,s) VALUES('p Multi',SYSDATETIME());
GO
EXEC dbo.p_dt_Multi;
GO 2
UPDATE #t SET e = SYSDATETIME() WHERE ID = 5;
GO
INSERT #t(q,s) VALUES('ad hoc Multi',SYSDATETIME());
GO
DECLARE @dt_m CHAR(10);
  SELECT @dt_m = dt.dt_
    FROM dbo.src AS o
    CROSS APPLY dbo.dt_Multi(o.create_date) AS dt
    ORDER BY o.[object_id];
GO 2
UPDATE #t SET e = SYSDATETIME() WHERE ID = 6;
GO
INSERT #t(q,s) VALUES('p Scalar',SYSDATETIME());
GO
EXEC dbo.p_dt_Scalar;
GO 2
UPDATE #t SET e = SYSDATETIME() WHERE ID = 7;
GO
INSERT #t(q,s) VALUES('ad hoc Scalar',SYSDATETIME());
GO
DECLARE @dt_sc CHAR(10);
  SELECT @dt_sc = dbo.dt_Scalar(create_date)
    FROM dbo.src
    ORDER BY [object_id];
GO 2
UPDATE #t SET e = SYSDATETIME() WHERE ID = 8;
GO
SELECT name = OBJECT_NAME(object_id), 
  execution_count,
  time_milliseconds = total_elapsed_time/1000
FROM sys.dm_exec_function_stats
WHERE database_id = DB_ID()
ORDER BY name;

SELECT name = OBJECT_NAME(object_id), 
  execution_count,
  time_milliseconds = total_elapsed_time/1000
FROM sys.dm_exec_procedure_stats
WHERE database_id = DB_ID()
ORDER BY name;

SELECT query = SUBSTRING([text],s,e), execution_count, time_milliseconds
FROM
(
  SELECT 
    t.[text], 
    s = s.statement_start_offset/2 + 1,
    e = COALESCE(NULLIF(s.statement_end_offset,-1),8000)/2,
    s.execution_count,
    time_milliseconds = s.total_elapsed_time/1000
  FROM sys.dm_exec_query_stats AS s
  OUTER APPLY sys.dm_exec_sql_text(s.[sql_handle]) AS t
  WHERE t.[text] LIKE N'%dt[_]%'
) AS x
ORDER BY query;
GO
SELECT query = q, 
    time_milliseconds = DATEDIFF(millisecond, s, e) 
  FROM #t 
  ORDER BY ID;
GO
DROP TABLE #t;
GO
USE master;
GO
DROP DATABASE fn;
GO